#!/usr/bin/env python
# -*- coding:utf-8 -*-
#
#       gmmctrl.py
#       
#       Copyright 2010 debian <debian@debian>
#       
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import os, sys, dbus, gobject, gtk,pygtk
from dbus.mainloop.glib import DBusGMainLoop
DBusGMainLoop(set_as_default=True)
sys_bus = dbus.SystemBus()
loop = gobject.MainLoop()
ssig=None
def enum_inputs(bus):
    '''liste les touches multimédia présentes sur le système'''
    obj = bus.get_object('org.freedesktop.Hal', '/org/freedesktop/Hal/Manager')
    hal_manager = dbus.Interface(obj, 'org.freedesktop.Hal.Manager')
    return hal_manager.FindDeviceByCapability("input.keys")
def read_buttons(bus):
    '''Connecte les touches multimédia à la fonction "showsignal"'''
    gobject.timeout_add (5000, loop.quit)
    for udi in enum_inputs(bus):
        obj = bus.get_object('org.freedesktop.Hal', udi)
        iface = dbus.Interface(obj, 'org.freedesktop.Hal.Device')
        iface.connect_to_signal("Condition", return_sig)
def return_sig(sender = None, destination = None):
    ''' retourne le signal de la touche multimedia appuyée '''
    if sender == 'ButtonPressed':
        global ssig
        ssig=destination
        loop.quit()
def read_confs():
    ''' Lecture du fichier de configuration de l'utilisateur '''
    try:
        HOME = os.environ['HOME']
        if os.path.isfile(HOME+'/.mmctrl.conf'):
            conf_file = open(HOME+'/.mmctrl.conf').readlines()
        elif os.path.isfile('/usr/share/python-dbus_mmctrl/mmctrl.conf'):
            conf_file = open(\
            '/usr/share/python-dbus_mmctrl/mmctrl.conf').readlines()

        confs = {}
        for line in conf_file:
            if not line.startswith('#') and not line == '\n':
                try:
                    confs[line.split('=')[0]] = line.split('=')[1]\
                    .split('\n')[0]
                except:
                    pass
    except:
        confs = {}
    return confs            

class ui():
    def write_confs(self,widget):
        ''' Lecture du fichier de configuration de l'utilisateur '''

        HOME = os.environ['HOME']
        try:
            os.remove(HOME+'/.mmctrl.conf')
        except:
            pass
        file=open(HOME+'/.mmctrl.conf','a')
        for key in self.conf.keys():
            if self.conf[key]!='':
                file.write(key+'='+self.conf[key]+'\n')
        file.close()
        
    def new_bind(self,widget):
        read_buttons(sys_bus)
        msg=gtk.Dialog()
        icon = self.window.render_icon(gtk.STOCK_PREFERENCES, gtk.ICON_SIZE_BUTTON)
        msg.set_icon(icon)
        msg.set_border_width(4)
        lbl=gtk.Label('Press a key or wait 5 secondes')
        lbl.show()
        msg.vbox.add(lbl)
        msg.show()
        loop.run()
        msg.destroy()
        if ssig!=None and not ssig in self.conf.keys():
            self.liststore.append([ssig, ''])
            self.conf[ssig]=''
        
    def del_bind(self,widget):
        self.selection = self.treeview.get_selection()
        #~ self.selection.set_mode(gtk.SELECTION_MULTIPLE)
        #~ print file
        model, paths, = self.selection.get_selected_rows()
        
        for path in paths:
            iter = model.get_iter(path)
            self.selection.select_iter(iter)
            self.liststore.remove(iter)
            
        
    def bind(self,widget,button,data=None):
        self.selection = self.treeview.get_selection()
        #~ self.selection.set_mode(gtk.SELECTION_MULTIPLE)
        #~ print file
        model, paths, = self.selection.get_selected_rows()
        
        self.selected_rule = None
        for path in paths:
            iter = model.get_iter(path)
            self.selection.select_iter(iter)
            (y, ) = model.get(iter, 0)
            
            
        msg=gtk.Dialog()
        entry=gtk.Entry()
        entry.show()
        msg.vbox.add(entry)
        msg.add_buttons(gtk.STOCK_OK,gtk.RESPONSE_OK,gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL)
        response=msg.run()
        if response==gtk.RESPONSE_OK:
            cmd=entry.get_text()
            self.conf[y]=cmd
            self.liststore.set_value(iter,1,cmd)
        msg.destroy()
        
    def delete_event(self, widget, event=None):
        gtk.main_quit()
        return False

    def make_pb(self, tvcolumn, cell, model, iter):
        stock = model.get_value(iter, 1)
        pb = self.treeview.render_icon(stock, gtk.ICON_SIZE_MENU, None)
        cell.set_property('pixbuf', pb)
        return

    def __init__(self):
    	
        
        buttons={"add":[0,gtk.STOCK_NEW,self.new_bind],
        "del":[0,gtk.STOCK_REMOVE,self.del_bind],
        "cancel":[1,gtk.STOCK_CLOSE,self.delete_event],
        "ok":[1,gtk.STOCK_OK,self.write_confs]}
        # Create a new window
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_title("Gmmctrl")
        icon = self.window.render_icon(gtk.STOCK_PREFERENCES, gtk.ICON_SIZE_BUTTON)
        self.window.set_icon(icon)
        self.window.set_size_request(300, 250)
        self.window.set_border_width(4)

        self.window.connect("delete_event", self.delete_event)
        box=gtk.VBox()
        
        title_box=gtk.HBox()
        title_box.show()
        logo=gtk.Image()
        logo.set_from_stock(gtk.STOCK_PREFERENCES,5)
        logo.set_size_request(64, 64)
        logo.show()
        lbl=gtk.Label()
        lbl.set_markup('<b><big> Gmmctrl</big></b>')
        lbl.show()
        title_box.pack_start(logo,False,True,2)
        title_box.pack_start(lbl,False,True,2)
        # create a liststore with one string column to use as the model
        box.pack_start(title_box,False,True,2)
        
        self.liststore = gtk.ListStore(str, str)

        # create the TreeView using liststore
        self.treeview = gtk.TreeView(self.liststore)
        box.pack_start(self.treeview,True,True,3)
        # create the TreeViewColumns to display the data
        self.tvcolumn = gtk.TreeViewColumn('Key')
        self.tvcolumn1 = gtk.TreeViewColumn('Action')
        
        # add columns to treeview
        self.treeview.append_column(self.tvcolumn)
        self.treeview.append_column(self.tvcolumn1)

        # create a CellRenderers to render the data
       
        self.cell = gtk.CellRendererText()
        self.conf=read_confs()
        
        for key in self.conf.keys():
            self.liststore.append([key.strip(),self.conf[key].strip()])


        # add the cells to the columns
        
        self.tvcolumn.pack_start(self.cell, True)
        self.tvcolumn1.pack_start(self.cell, True)

        # set the cell attributes to the appropriate liststore column
                
        self.tvcolumn.set_attributes(self.cell, text=0)
        self.tvcolumn1.set_attributes(self.cell, text=1)

        # make treeview searchable
        self.treeview.set_search_column(0)

        # Allow sorting on the column
        self.tvcolumn.set_sort_column_id(0)
        # sort the rows in column
        self.liststore.set_sort_column_id(0,gtk.SORT_ASCENDING)

        # Allow drag and drop reordering of rows
        self.treeview.set_reorderable(True)
        self.treeview.connect("row-activated",self.bind)
        self.treeview.show()
        
        
        self.window.add(box)
        
        hbox=gtk.HBox()
        hbox.show()
        box.pack_start(hbox,False,True,2)
        for id in buttons.keys():
            if buttons[id][0]==0:
                    
                btn=gtk.Button("",buttons[id][1])
                btn.show()
                hbox.pack_start(btn,False,True,2)
                btn.connect("clicked",buttons[id][2])
        hbox=gtk.HBox()
        hbox.show()
        box.pack_start(hbox,False,True,2)
        for id in buttons.keys():
            if buttons[id][0]==1:
                    
                btn=gtk.Button("",buttons[id][1])
                btn.show()
                hbox.pack_end(btn,False,True,2)
                btn.connect("clicked",buttons[id][2])
            

        self.window.show_all()

def main():
    ui()
    gtk.main()
if __name__ == '__main__': main()
